/*
 * file StpActivity.java
 *
 * Licensed Materials - Property of IBM
 * Restricted Materials of IBM 
 *
 * com.ibm.rational.wvcm.stp.StpActivity
 *
 * (C) Copyright IBM Corporation 2007, 2009.  All Rights Reserved.
 * Note to U.S. Government Users Restricted Rights:  Use, duplication or 
 * disclosure restricted by GSA ADP  Schedule Contract with IBM Corp.
 */

package com.ibm.rational.wvcm.stp;

import static com.ibm.rational.wvcm.stpex.StpExBase.PROPERTY_NAMESPACE;

import javax.wvcm.WvcmException;
import javax.wvcm.PropertyNameList.PropertyName;

import com.ibm.rational.wvcm.stp.cc.CcActivity;
import com.ibm.rational.wvcm.stp.cc.CcTypeBase.Scope;
import com.ibm.rational.wvcm.stp.cq.CqQuery;
import com.ibm.rational.wvcm.stp.cq.CqRecord;
import com.ibm.rational.wvcm.stpex.StpExEnumeration;
import com.ibm.rational.wvcm.stpex.StpExEnumerationBase;

/**
 * <p>
 * StpActivity is an interface for a proxy which may represent one of these
 * three cases:
 * </p>
 * <ul>
 * <li>A UCM-enabled CQ record not bound to a UCM activity, or</li>
 * <li>A UCM activity not bound to a CQ record, or</li>
 * <li>A UCM-enabled CQ record-UCM activity pair (bound together)</li>
 * </ul>
 * <p>
 * It serves two purposes:
 * </p>
 * <ul>
 * <li>It provides access to properties which are common to all of the above
 * three cases</li>
 * <li>It provides a means to access the CcActivity and/or CqRecord proxies for
 * the persistent UCM activity or CQ record resources involved in the above
 * three cases</li>
 * </ul>
 * <p>
 * Although this interface extends StpResource, it is an abstraction. There is
 * no factory method for constructing an instance of StpActivity, specifically.
 * Proxies that implement this interface may be constructed only by constructing
 * a CcActivity or a CqRecord. The location of an StpActivity will be that of
 * a CcActivity or a CqRecord.
 */
public interface StpActivity extends StpResource {

    enum CqUcmIntegrationState implements StpExEnumeration {

        /**
         * This is an unbound UCM activity in a non-ClearQuest-enabled project.
         */
        UNBOUND_CC_ACTIVITY,

        /**
         * This is an unbound ClearQuest record.
         */
        UNBOUND_CQ_RECORD,

        /**
         * This is a UCM-enabled CQ record and a UCM activity that are bound to
         * each other.
         */
        BOUND_CC_ACTIVITY_CQ_RECORD_PAIR;
    }

    /**
     * This activity's CQ-UCM integration state
     */
    PropertyName<CqUcmIntegrationState> CQ_UCM_INTEGRATION_STATE =
        new PropertyName<CqUcmIntegrationState>(PROPERTY_NAMESPACE,
                                                "cq-ucm-integration-state");

    /**
     * Returns the value of this proxy's {@link #CQ_UCM_INTEGRATION_STATE}
     * property.
     * 
     * @return this activity's CQ-UCM integration state
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    CqUcmIntegrationState getCqUcmIntegrationState() throws WvcmException;

    /**
     * The ClearCase activity to which this StpActivity is bound. In
     * ClearQuest-enabled UCM projects, all UCM activities are bound and the
     * value of this property will always be non-null. In non-CQ-enabled
     * projects, the value of this property will always be <b>null</b>.
     */
    PropertyName<CcActivity> BOUND_CC_ACTIVITY =
        new PropertyName<CcActivity>(PROPERTY_NAMESPACE, "bound-cc-activity");

    /**
     * Returns the value of this proxy's {@link #BOUND_CC_ACTIVITY} property.
     * 
     * @return this activity's bound CQ record, or <b>null</b> if thi activity
     *         is not bound to a CcActivity
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    CcActivity getBoundCcActivity() throws WvcmException;

    /**
     * The ClearQuest record to which this UCM activity is bound. In
     * ClearQuest-enabled UCM projects, all UCM activities are bound and the
     * value of this property will always be non-null. In non-CQ-enabled
     * projects, the value of this property will always be <b>null</b>.
     */
    PropertyName<CqRecord> BOUND_CQ_RECORD =
        new PropertyName<CqRecord>(PROPERTY_NAMESPACE, "bound-cq-record");

    /**
     * Returns the value of this proxy's {@link #BOUND_CQ_RECORD} property.
     * 
     * @return this activity's bound CQ record, or <b>null</b> if activity is
     *         not bound to a CQ record.
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    CqRecord getBoundCqRecord() throws WvcmException;

    /**
     * A descriptive string - not necessarily unique - used to identify
     * this activity when it appears in a GUI or in a report.
     */
    PropertyName<String> HEADLINE =
        new PropertyName<String>(PROPERTY_NAMESPACE, "headline");

    /**
     * Returns the value of this proxy's {@link #HEADLINE} property.
     * 
     * @return A String containing the description of this StpActivity
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    String getHeadline() throws WvcmException;

    /**
     * Defines a new value for the {@link #HEADLINE} property of this proxy.
     * 
     * @param headline The new headline string. Must not be <b>null</b>.
     */
    void setHeadline(String headline);

    /**
     * @deprecated 
     * Use {@link #ID_SELECTOR} for a unique string to identify
     * this activity.
     * <br>
     * Use {@link javax.wvcm.Resource#DISPLAY_NAME} for the activity name.
     */
    PropertyName<String> ID =
        new PropertyName<String>(PROPERTY_NAMESPACE, "id");

    /**
     * @deprecated
     */
    String getId() throws WvcmException;
    
    /**
     * A unique string to identify this activity.
     */
    PropertyName<String> ID_SELECTOR =
        new PropertyName<String>(PROPERTY_NAMESPACE, "id-selector");
    
    /**
     * Returns the value of this proxy's {@link #ID_SELECTOR} property.
     * 
     * @return A String containing the unique identifier for this StpActivity
     * @throws WvcmException if this proxy doesn't define a value for this
     *             property.
     */
    String getIdSelector() throws WvcmException;
}